package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class TranslatableComponentImpl extends AbstractComponent implements TranslatableComponent {
   private final String key;
   private final @Nullable String fallback;
   private final List<TranslationArgument> args;

   static TranslatableComponent create(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String key, final @Nullable String fallback, final @NotNull ComponentLike @NotNull [] args) {
      Objects.requireNonNull(args, "args");
      return create(children, style, key, fallback, Arrays.asList(args));
   }

   static TranslatableComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final @NotNull String key, final @Nullable String fallback, final @NotNull List<? extends ComponentLike> args) {
      return new TranslatableComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(key, "key"), fallback, asArguments(args));
   }

   TranslatableComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String key, final @Nullable String fallback, final @NotNull List<TranslationArgument> args) {
      super(children, style);
      this.key = key;
      this.fallback = fallback;
      this.args = args;
   }

   public @NotNull String key() {
      return this.key;
   }

   public @NotNull TranslatableComponent key(final @NotNull String key) {
      return (TranslatableComponent)(Objects.equals(this.key, key) ? this : create(this.children, this.style, key, this.fallback, this.args));
   }

   /** @deprecated */
   @Deprecated
   public @NotNull List<Component> args() {
      return ComponentLike.asComponents(this.args);
   }

   public @NotNull List<TranslationArgument> arguments() {
      return this.args;
   }

   public @NotNull TranslatableComponent arguments(final @NotNull ComponentLike... args) {
      return create(this.children, this.style, this.key, this.fallback, args);
   }

   public @NotNull TranslatableComponent arguments(final @NotNull List<? extends ComponentLike> args) {
      return create(this.children, this.style, this.key, this.fallback, args);
   }

   public @Nullable String fallback() {
      return this.fallback;
   }

   public @NotNull TranslatableComponent fallback(final @Nullable String fallback) {
      return create(this.children, this.style, this.key, fallback, this.args);
   }

   public @NotNull TranslatableComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.key, this.fallback, this.args);
   }

   public @NotNull TranslatableComponent style(final @NotNull Style style) {
      return create(this.children, style, this.key, this.fallback, this.args);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof TranslatableComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         TranslatableComponent that = (TranslatableComponent)other;
         return Objects.equals(this.key, that.key()) && Objects.equals(this.fallback, that.fallback()) && Objects.equals(this.args, that.arguments());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.key.hashCode();
      result = 31 * result + Objects.hashCode(this.fallback);
      result = 31 * result + this.args.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   @NotNull
   public TranslatableComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static List<TranslationArgument> asArguments(final @NotNull List<? extends ComponentLike> likes) {
      if (likes.isEmpty()) {
         return Collections.emptyList();
      } else {
         List<TranslationArgument> ret = new ArrayList(likes.size());

         for(int i = 0; i < likes.size(); ++i) {
            ComponentLike like = (ComponentLike)likes.get(i);
            if (like == null) {
               throw new NullPointerException("likes[" + i + "]");
            }

            if (like instanceof TranslationArgument) {
               ret.add((TranslationArgument)like);
            } else if (like instanceof TranslationArgumentLike) {
               ret.add((TranslationArgument)Objects.requireNonNull(((TranslationArgumentLike)like).asTranslationArgument(), "likes[" + i + "].asTranslationArgument()"));
            } else {
               ret.add(TranslationArgument.component(like));
            }
         }

         return Collections.unmodifiableList(ret);
      }
   }

   static final class BuilderImpl extends AbstractComponentBuilder<TranslatableComponent, TranslatableComponent.Builder> implements TranslatableComponent.Builder {
      private @Nullable String key;
      private @Nullable String fallback;
      private List<TranslationArgument> args = Collections.emptyList();

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull TranslatableComponent component) {
         super(component);
         this.key = component.key();
         this.args = component.arguments();
         this.fallback = component.fallback();
      }

      @NotNull
      public TranslatableComponent.@NotNull Builder key(final @NotNull String key) {
         this.key = key;
         return this;
      }

      @NotNull
      public TranslatableComponent.@NotNull Builder arguments(final @NotNull ComponentLike... args) {
         Objects.requireNonNull(args, "args");
         return args.length == 0 ? this.arguments(Collections.emptyList()) : this.arguments(Arrays.asList(args));
      }

      @NotNull
      public TranslatableComponent.@NotNull Builder arguments(final @NotNull List<? extends ComponentLike> args) {
         this.args = TranslatableComponentImpl.asArguments((List)Objects.requireNonNull(args, "args"));
         return this;
      }

      @NotNull
      public TranslatableComponent.@NotNull Builder fallback(final @Nullable String fallback) {
         this.fallback = fallback;
         return this;
      }

      public @NotNull TranslatableComponent build() {
         if (this.key == null) {
            throw new IllegalStateException("key must be set");
         } else {
            return TranslatableComponentImpl.create(this.children, this.buildStyle(), this.key, this.fallback, this.args);
         }
      }
   }
}
